@@ -1,7 +1,7 @@
 /*
  * AMBA Plug'n Play vendor and device ID definitions.
  *
- * COPYRIGHT (c) 2024.
+ * COPYRIGHT (c) 2025.
  * Frontgrade Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
  * Any changes/bug-fixes should be made in the script, then rerun the script
  * to create a new file.
  *
- * Created from GRLIB revision 4293
+ * Created from GRLIB revision 4297
  */
 
 #ifndef _ambapp_ids_h_
@@ -63,8 +63,10 @@
 #define VENDOR_SEMIBLOCKS    0x28
 #define VENDOR_NEC           0x29
 #define VENDOR_HONEYWELL     0x2a
+#define VENDOR_ST            0x2b
 #define VENDOR_AITECH        0x30
 #define VENDOR_S3            0x31
+#define VENDOR_TTTECH        0x40
 #define VENDOR_TAS           0xa5
 #define VENDOR_RECORE        0xa6
 #define VENDOR_AAC           0xaa
@@ -295,6 +297,16 @@
 #define GAISLER_SGMII_VCU118 0x0d4
 #define GAISLER_APBUART_16550 0x0d5
 #define GAISLER_IOPMP        0x0d6
+#define GAISLER_EFPGA        0x0d7
+#define GAISLER_GRISOC_MB_EXT 0x0d8
+#define GAISLER_GRISOC_MB_INT 0x0d9
+#define GAISLER_GRISOC_BOOT  0x0da
+#define GAISLER_GRISOC_CRYPTO 0x0db
+#define GAISLER_GRISOC_SRNG  0x0dc
+#define GAISLER_GRPPSTIMER   0x0dd
+#define GAISLER_GRLSEDC      0x0de
+#define GAISLER_MDIO_CTRL    0x0df
+#define GAISLER_OTP_WRPR     0x0e0
 
 
 /* ESA
@@ -420,6 +432,20 @@
 #define NASA_EP32            0x001
 
 
+/* ST
+ * ST Microelectronics device ID defines
+ */
+#define GAISLER_AHBMST_EM    0x040
+#define GAISLER_TEST_1X2     0x078
+#define ST_MIPHY             0x001
+
+
+/* TTTECH
+ * TTTECH device ID defines
+ */
+#define TTTECH_SWITCH        0x001
+
+
 /* TAS
  * Thales Alenia Space device ID defines
  */
@@ -558,6 +584,7 @@
 #define XILINX_ML605         0x0605
 #define LEON_LATTICE_NEXUS   0x0800
 #define GAISLER_GR740MINI    0x0801
+#define FRONTGRADE_SBC8      0x0802
 #define XILINX_AC701         0xa701
 #define XILINX_KC705         0xa705
 #define XILINX_VC707         0xa707
