@@ -26,6 +26,8 @@
 
 #ifdef CONFIG_WATCHDOG
 
+#define WATCHDOG_TIMEOUT_NOT_SET 0
+
 enum ping_id {
     PING_ID_PROM_NO,
     PING_ID_UART,
@@ -33,10 +35,18 @@ enum ping_id {
     PING_ID_MAX
 };
 
+struct watchdog_prop_conf {
+	unsigned int user_timeout_sec;
+	unsigned int hw_timeout_ms;
+	const char *compatible;
+};
+
 void watchdog_init(void);
 void watchdog_ping(enum ping_id id);
 void watchdog_set_timeout(unsigned int waittime_ms);
 
+struct watchdog_prop_conf *watchdog_get_prop_conf(unsigned int ambapp_device_id,
+						  unsigned int core_idx);
 #endif
 
 #endif
