@@ -206,3 +206,12 @@ char *no_nextprop(int node, char *name)
 	else
 		return NULL;
 }
+
+void node_add_child(struct node *node, struct node *child)
+{
+	struct node **np = &node->child;
+
+	while (*np)
+		np = &((*np)->sibling);
+	*np = child;
+}
