@@ -69,8 +69,6 @@ struct watchdog {
 
 /* Hardware timeout for the GR740 PLL watchdog  */
 #define GPTIMER_WATCHDOG_GR740_HW_TIMEOUT 2000
-/* Bit 0-2 mask for reading # timers from the conf register */
-#define GPTIMER_CONF_TIMERS_MASK 0x7
 /* Reload bit of the timer's ctrl register */
 #define GPTIMER_CTRL_RD 4
 
