@@ -56,6 +56,7 @@
 enum matchtype {
 	MATCHTYPE_CORE = 0,
 	MATCHTYPE_BUS,
+	MATCHTYPE_ROOT,
 	MATCHTYPE_NONE,
 };
 
@@ -245,6 +246,7 @@ enum tag {
 	TAG_INT,
 	TAG_COREHANDLE,
 	TAG_STRING,
+	TAG_MATCHROOT,
 	NUMBER_OF_TAGS		/* MUST be last one */
 };
 
@@ -347,7 +349,8 @@ static struct node *get_parent_from_stack(struct scanpriv *priv,
 {
 	enum tag tag = get_upper_tag(priv, currframe);
 
-	if (tag != TAG_MATCHCORE && tag != TAG_MATCHBUS && tag != TAG_ADDNODE) {
+	if (tag != TAG_MATCHCORE && tag != TAG_MATCHBUS && tag != TAG_ADDNODE &&
+	    tag != TAG_MATCHROOT) {
 		fprintf(stderr, IERR "cannot get parent from type %d\n", tag);
 		return NULL;
 	}
@@ -679,6 +682,15 @@ static struct match *new_bus_match(struct scanpriv *priv, enum action action,
 	return m;
 }
 
+static struct match *new_root_match(struct scanpriv *priv, enum action action,
+				    const char *bus)
+{
+	struct match *m;
+
+	m = new_match(priv, MATCHTYPE_ROOT, action);
+
+	return m;
+}
 
 /* ============================================================ */
 /* Documentation */
@@ -786,6 +798,10 @@ static void doc_matchbus(struct taginfo *ti)
 	printf("   Matches a bus. A mandatory \"%s\" attribute names the bus number to match\n", MATCH_BUS);
 }
 
+static void doc_matchroot(struct taginfo *ti)
+{
+	printf("   Matches the root node.\n");
+}
 
 static void doc_print()
 {
@@ -901,6 +917,9 @@ static int start_match(struct scanpriv *priv, struct stackframe *frame,
 	} else if (frame->ti->tag == TAG_MATCHBUS) {
 		matchtype = MATCHTYPE_BUS;
 		action = ACTION_MATCH;
+	} else if (frame->ti->tag == TAG_MATCHROOT) {
+		matchtype = MATCHTYPE_ROOT;
+		action = ACTION_MATCH;
 	} else {
 		fprintf(stderr, IERR "Unknown match tag type %d\n",
 			frame->ti->tag);
@@ -932,6 +951,8 @@ static int start_match(struct scanpriv *priv, struct stackframe *frame,
 			return 1;
 		}
 		m = new_bus_match(priv, action, bus);
+	} else if (matchtype == MATCHTYPE_ROOT) {
+		m = new_root_match(priv, action, bus);
 	}
 
 	if (!m)
@@ -1161,6 +1182,7 @@ static int init_tags(void) {
 	valid_child(ti, TAG_MATCHCORE);
 	valid_child(ti, TAG_DELCORE);
 	valid_child(ti, TAG_MATCHBUS);
+	valid_child(ti, TAG_MATCHROOT);
 
 	/*
 	 * Several tags uses the same tag parse function start_match, but has
@@ -1180,6 +1202,10 @@ static int init_tags(void) {
 	set_doc_function(ti, doc_matchbus);
 	valid_child(ti, TAG_ADDNODE);
 
+	ti = init_tag(TAG_MATCHROOT, "match-root", start_match, NULL);
+	set_doc_function(ti, doc_matchroot);
+	valid_child(ti, TAG_ADDNODE);
+
 	ti = init_tag(TAG_CORELABEL, "corelabel", start_corelabel, NULL);
 	set_doc_function(ti, doc_corelabel);
 
@@ -1635,6 +1661,9 @@ static int generate_matchline(FILE *fout, struct scanpriv *priv,
 	} else if (m->matchtype == MATCHTYPE_BUS) {
 		FPRINTF(fout, "0, 0, 0, 0, %s, ", m->bus);
 		flags1 = "XMLMATCH_BUS";
+	} else if (m->matchtype == MATCHTYPE_ROOT) {
+		FPRINTF(fout, "0, 0, 0, 0, 0, ");
+		flags1 = "XMLMATCH_ROOT";
 	} else {
 		fprintf(stderr, IERR "No matchline generation for matchtype %d\n",
 			m->matchtype);
@@ -1663,7 +1692,7 @@ static int generate_matches(FILE *fout, struct scanpriv *priv)
 
 	FPRINTF(fout, "/*\n");
 	FPRINTF(fout, " * Each entry consists of:\n");
-	FPRINTF(fout, " *   vendor, device, index, coreindex, flags,\n");
+	FPRINTF(fout, " *   vendor, device, index, coreindex, bus, flags,\n");
 	FPRINTF(fout, " *   index of placeholder node in xml_nodes[], \n");
 	FPRINTF(fout, " *   first corehandle to fix up,\n");
 	FPRINTF(fout, " *   # of corehandles to fix up\n");
