@@ -6,6 +6,9 @@
 #include <startup.h>
 #include <ambapp.h>
 #include <prom_watchdog.h>
+#ifdef CONFIG_HRTIMERS
+#include <cpucounter.h>
+#endif
 
 static unsigned long startup_malloc_end = (unsigned long)&_startup_heap_end;
 
@@ -77,6 +80,9 @@ startup_main(void)
 	watchdog_init();
 #endif
 
+#ifdef CONFIG_HRTIMERS
+	LEON_UP_COUNTER_INIT();
+#endif
 	/* Create PROM nodes of all AMBA cores */
 	ambapp_nodes_init(opprom_get_ambapp0());
 
