@@ -17,6 +17,9 @@
 #ifdef CONFIG_WATCHDOG
 #include <prom_watchdog.h>
 #endif
+#ifdef CONFIG_HRTIMERS
+#include <cpucounter.h>
+#endif
 
 #define REMOVE_NODE 1
 
@@ -24,6 +27,8 @@
 #define GR740_REV0_SYSTEMID 0x07401038
 #define GR740_REV1_SYSTEMID 0x07401039
 
+#define GR740_DEBUG_UART 0xff900000
+
 int amba_systemid_is_gr740(void)
 {
 	return (amba_systemid == GR740_REV0_SYSTEMID) ||
@@ -70,6 +75,7 @@ void leon_init(ambapp_core *core, int level)
 
 static int irqmp_initied = 0;
 static int eirqs[16];
+static int irqmp_ncpus = 0;
 
 static void irqmp_add_eirq(struct irqmp_regs *cregs)
 {
@@ -89,7 +95,7 @@ static void irqmp_add_eirq(struct irqmp_regs *cregs)
 
 int irqmp_dev(ambapp_dev *dev)
 {
-	int c, icsel, nctrl;
+	int c, icsel, nctrl, ncpu;
 	ambapp_apbdev *apbslv;
 	struct irqmp_regs *regs, *cregs;
 
@@ -99,6 +105,16 @@ int irqmp_dev(ambapp_dev *dev)
 		/* Only process the first IRQMP core */
 		irqmp_initied = 1;
 		irqmp = regs;
+		/* Get Number of CPUs */
+		#ifndef CONFIG_SMP
+			ncpu = 1;
+		#else
+			ncpu = BYPASS_LOAD_PA(&irqmp->mpstatus);
+			ncpu = ((ncpu >> IRQMP_STATUS_CPUNR) & 0xf) + 1;
+			if (ncpu > CONFIG_MAX_CPUS)
+				ncpu = CONFIG_MAX_CPUS;
+		#endif
+		irqmp_ncpus= ncpu;
 	}
 
 	/* Record eirqs, so that grgpio can avoid collisions */
@@ -118,7 +134,11 @@ int irqmp_dev(ambapp_dev *dev)
 	return 0;
 }
 
-void node_add_child(struct node *node, struct node *child);
+int irqmp_numcpus(void)
+{
+	return irqmp_ncpus;
+}
+
 int node_has_child(struct node *node, const char *child_name, int name_len, struct node **child);
 
 void ambapp_prop_init(struct prop_std *buf, const char *prop_str);
@@ -129,9 +149,16 @@ int ambapp_prop_uintarray(struct prop_std *buf, const char *prop_str,
 int ambapp_prop_str_array(struct prop_std *buf, const char *prop_str,
 			  const char *val_array[]);
 int ambapp_prop_empty(struct prop_std *buf, const char *prop_str);
+int ambapp_prop_intptr(struct prop_std *buf, const char *prop_str, int *value,
+		       int len);
 
 int gptimer_inited = 0;
 
+#ifdef CONFIG_HRTIMERS
+/* Incremented by gptimer_init for each core */
+int gptimer_core_cnt = 0;
+#endif
+
 /* Fixup IRQ of GPTIMER APBSLV interface */
 int gptimer_apb_fixup(ambapp_dev *dev)
 {
@@ -225,17 +252,376 @@ void node_add_watchdog(struct node *parent, unsigned int ambapp_device_id,
 	/* Finally, add the watchdog node to the parent node */
 	node_add_child(parent, n);
 }
+#endif
+
+#ifdef CONFIG_HRTIMERS
+
+/* Holds the property values for the driver's bindings */
+struct hrtimers_prop_conf {
+	int timer_offset;
+	int timer_len;
+	int cpu_offset;
+	int cs_timer_index;
+};
+
+/* Initialized by the hrtimers_init */
+static struct hrtimers_prop_conf hrtimers_prop_conf;
+
+/*
+ * Here follows the different timer allocation strategies that are supported:
+ *  TMR_ALLOC_SETUP_CE:
+ *    Setup clock event devices on subtimers for each GPTIMER core for as
+ *    many CPUs in the system. Care have to be taken for GPTIMER0 where the
+ *    last timer is reserved for GPTIMER Watchdog and the for GPTIMERS without
+ *    separate interrupts only one timer can be setup.
+ *  TMR_ALLOC_SETUP_CS:
+ *    Setup a clock source either on the same core as the clock events or on a
+ *    separate GPTIMER core. Care have to be taken for GPTIMER0 where the
+ *    last timer is reserved for GPTIMER Watchdog.
+ */
+
+#define TMR_ALLOC_SETUP_CE (1 << 0)
+#define TMR_ALLOC_SETUP_CS (1 << 1)
+
+/*
+ * Setup by hrtimers_init and is then assigned:
+ * - TMR_ALLOC_SETUP_CE
+ * Optionally masked with:
+ * - TMR_ALLOC_SETUP_CS
+ */
+static int tmr_alloc_alt;
+/* Number of configured clock event devices */
+static int clkevt_dev_cnt;
+
+/* The result values */
+enum hrtimers_errno {
+	HRTIMER_RES_OK,
+	HRTIMER_RES_CS_SETUP_FAILED,
+	HRTIMER_RES_SEP_GPTIMERS_FAILED,
+	HRTIMER_RES_INVALID_CONFIG,
+};
+
+/* The result of the hrtimer operations */
+static enum hrtimers_errno hrtimers_res = HRTIMER_RES_OK;
+
+/* Returns true (1) if the hrtimer operations were successful. */
+int hrtimers_res_ok(void)
+{
+	return hrtimers_res == HRTIMER_RES_OK;
+}
+
+/* Set the pre-scaler and scaler reload to the same as GPTIMER0 */
+static inline void hrtimers_set_prescaler(ambapp_core *core, int numtmr)
+{
+	struct gptimer_regs *regs;
+	unsigned int scaler;
+
+	if (numtmr == 0 || !core->apbslv || !gptimer_inited)
+		return;
+
+	/* Setup the prescaler (use same as for gptimer0) */
+	regs = (struct gptimer_regs *)core->apbslv->address;
+	scaler = BYPASS_LOAD_PA(&gptimer->scaler_reload);
+	BYPASS_STORE_PA(&regs->scaler_reload, scaler);
+	BYPASS_STORE_PA(&regs->scaler, scaler);
+}
+
+static int hrtimers_gptimer_probe(ambapp_core *core, int numtmr)
+{
+	struct gptimer_regs *regs;
+	unsigned int config;
+	unsigned int subtimers;
+	int ncpu;
+	int timers_left;
+	int timer_len;
+
+	/* Nothing to do */
+	if (!tmr_alloc_alt)
+		return -1;
+
+	if (!core || !core->apbslv)
+		goto err_invalid_configuration;
+
+	regs = (struct gptimer_regs *)core->apbslv->address;
+	config = BYPASS_LOAD_PA(&regs->config);
+
+	ncpu = irqmp_numcpus();
+	if (!ncpu)
+		goto err_invalid_configuration;
+
+	/* How many timers are left ? */
+	timers_left = ncpu - clkevt_dev_cnt;
+	if (timers_left < 0)
+		goto err_invalid_configuration;
+
+	/* The number of timers we would like to setup on this core */
+	timer_len = timers_left;
+
+	/* Now check that there is enough sub-timers */
+	subtimers = config & GPTIMER_CONF_TIMERS_MASK;
+	if (!subtimers)
+		goto err_invalid_configuration;
+
+	/* The last timer of GPTIMER0 is reserved to the GPTIMER watchdog. */
+	if (numtmr == 0)
+		subtimers--;
+
+	if (subtimers <= 0)
+		goto err_invalid_configuration;
+	/*
+	 * Setup all clockevent devices on the GPTIMER.
+	 * If the GPTIMER supports separate interrupts then we use as many
+	 * subtimers as we can.
+	 */
+	if (tmr_alloc_alt & TMR_ALLOC_SETUP_CE) {
+		if (config & GPTIMER_CONF_SEP_IRQ) {
+			if (timer_len > subtimers)
+				timer_len = subtimers;
+		} else {
+			timer_len = 1;
+		}
+
+		/* Initialize property values */
+		/* FIXME: ampopts not considered as for now */
+		hrtimers_prop_conf.timer_offset = 0;
+		hrtimers_prop_conf.timer_len = timer_len;
+		hrtimers_prop_conf.cpu_offset = clkevt_dev_cnt;
+
+	} else {
+		hrtimers_prop_conf.timer_offset = 0;
+		hrtimers_prop_conf.timer_len = 0;
+		hrtimers_prop_conf.cpu_offset = 0;
+		timer_len = 0;
+	}
+
+	/*
+	 * Setup clock event source as long as there is one available subtimer
+	 */
+	hrtimers_prop_conf.cs_timer_index = -1;
+	if ((tmr_alloc_alt & TMR_ALLOC_SETUP_CS) && timer_len < subtimers)
+		hrtimers_prop_conf.cs_timer_index = timer_len;
+
+	DBG_PRINTF1("HRTIMER conf (gptimer%d):\n", numtmr);
+	DBG_PRINTF2("   subtimers: %d, clkevt_dev_cnt: %d\n",
+		    subtimers, clkevt_dev_cnt);
+	DBG_PRINTF2("   cpu_offset: %d, timer_len: %d\n",
+		    hrtimers_prop_conf.cpu_offset,
+		    hrtimers_prop_conf.timer_len);
+	DBG_PRINTF3("   timer_offset: %d, cs_timer_index: %d, ncpu: %d\n",
+		    hrtimers_prop_conf.timer_offset,
+		    hrtimers_prop_conf.cs_timer_index, ncpu);
+
+	return 0;
+
+err_invalid_configuration:
+	hrtimers_res = HRTIMER_RES_INVALID_CONFIG;
+	return -1;
+}
+
+/* Adds the hrtimer related properties */
+static int hrtimers_add_timer_prop(ambapp_core *core, struct prop_std *buf,
+				   int index, int numtmr)
+{
+	const char *compatible = prop_str_compatible_timer_ce;
+
+	switch (index) {
+	case 0:
+		if (hrtimers_gptimer_probe(core, numtmr) < 0)
+			return 0;
+
+		hrtimers_set_prescaler(core, numtmr);
+		return ambapp_prop_intptr(buf, prop_str_clocks,
+					  (int*)&node_gptimer0_clk,
+					  sizeof(node_gptimer0_clk));
+	case 1:
+		return ambapp_prop_int(buf, prop_str_tmr_offset,
+				       hrtimers_prop_conf.timer_offset);
+	case 2:
+		return ambapp_prop_int(buf, prop_str_tmr_len,
+				       hrtimers_prop_conf.timer_len);
+	case 3:
+		return ambapp_prop_int(buf, prop_str_cpu_offset,
+				       hrtimers_prop_conf.cpu_offset);
+	case 4:
+		/*
+		 * Update compatible in case we setup both clocksource and
+		 * clockevents for this GPTIMER core
+		 */
+		if (hrtimers_prop_conf.cs_timer_index >= 0)
+			compatible = prop_str_compatible_timer;
+
+		return ambapp_prop_str(buf, prop_str_compatible, compatible);
+	case 5:
+		if (hrtimers_prop_conf.cs_timer_index >= 0) {
+			/* Clock source has been setup, clear action */
+			tmr_alloc_alt &= ~TMR_ALLOC_SETUP_CS;
+			return ambapp_prop_int(buf, prop_str_cs_tmr_idx,
+					       hrtimers_prop_conf.cs_timer_index);
+		}
+		/* Fallthrough */
+	case 6:
+		/* The GPTIMER core has been configured. */
+		clkevt_dev_cnt += hrtimers_prop_conf.timer_len;
+		break;
+	default:
+		break;
+	}
+
+	return 0;
+}
+
+/* Setup all clock event devices */
+static int hrtimers_setup_ce(ambapp_core *core, struct prop_std *buf, int index,
+			     int numtmr)
+{
+	int ret = 0;
+
+	ret = hrtimers_add_timer_prop(core, buf, index, numtmr);
+	if (ret == 0 && (irqmp_numcpus() == clkevt_dev_cnt))
+		tmr_alloc_alt &= ~TMR_ALLOC_SETUP_CE;
+
+	return ret;
+}
+
+/* Setup the timer allocation for the GPTIMER clocksource */
+static int hrtimers_setup_cs(ambapp_core *core, struct prop_std *buf,
+				     int index, int numtmr)
+{
+	switch (index) {
+	case 0:
+		if (hrtimers_gptimer_probe(core, numtmr) < 0)
+			return 0;
+
+		if (hrtimers_prop_conf.cs_timer_index == -1)
+			return 0;
+
+		hrtimers_set_prescaler(core, numtmr);
+		return ambapp_prop_intptr(buf, prop_str_clocks,
+					  (int*)&node_gptimer0_clk,
+					  sizeof(node_gptimer0_clk));
+	case 1:
+		return ambapp_prop_str(buf, prop_str_compatible,
+				       prop_str_compatible_timer_cs);
+	case 2:
+		tmr_alloc_alt &= ~TMR_ALLOC_SETUP_CS;
+		return ambapp_prop_int(buf, prop_str_cs_tmr_idx, 0);
+
+	default:
+		break;
+	}
+	return 0;
+}
+
+/* Find out how to allocate the GPTIMERS in this system */
+void hrtimers_init()
+{
+	tmr_alloc_alt = TMR_ALLOC_SETUP_CE;
+
+	/*
+	 * Check if a GPTIMER based clocksource should be setup:
+	 * - If its possible to use the CPU up-counter in LEON then prefer it.
+	 * - Otherwise setup a GPTIMER clocksource on the first found core with
+	 *   enough timers.
+	 */
+	if (LEON_UP_COUNTER_AVAIL())
+		goto done;
+
+	/* Setup a GPTIMER clocksource */
+	tmr_alloc_alt |= TMR_ALLOC_SETUP_CS;
+
+done:
+	return;
+}
+
+int gptimer_fixup_prop(ambapp_core *core, struct prop_std *buf, int index)
+{
+	/* Use static as core subindex is not generally available. */
+	static int numtmr = -1;
+	int ret = 0;
+
+	if (numtmr == -1)
+		hrtimers_init();
+
+	if (index == 0)
+		numtmr++;
+
+	if (hrtimers_res_ok()) {
+		if (tmr_alloc_alt & TMR_ALLOC_SETUP_CE)
+			ret = hrtimers_setup_ce(core, buf, index, numtmr);
+		else if (tmr_alloc_alt & TMR_ALLOC_SETUP_CS)
+			ret = hrtimers_setup_cs(core, buf, index, numtmr);
+	}
+
+	/* Sanity check when all gptimer cores have been setup */
+	if (ret == 0 && numtmr == (gptimer_core_cnt - 1)) {
+		if (tmr_alloc_alt & TMR_ALLOC_SETUP_CE) {
+			DBG_PRINTF0("HRTIMER: Failed to setup clock event bindings!\n");
+			hrtimers_res = HRTIMER_RES_INVALID_CONFIG;
+		}
+
+		if (tmr_alloc_alt & TMR_ALLOC_SETUP_CS) {
+			DBG_PRINTF0("HRTIMER: Failed to setup clock source bindings!\n");
+			hrtimers_res = HRTIMER_RES_INVALID_CONFIG;
+		}
+	}
+
+	return ret;
+}
+
+#else
+#define gptimer_fixup_prop NULL
+#endif
+
+#if defined(CONFIG_WATCHDOG) || defined(CONFIG_HRTIMERS)
+
+#ifdef CONFIG_HRTIMERS
+
+#define HRTIMER_MAX_ROLLBACK_NODES 32
+static struct node* hrtimer_rollback_nodes[HRTIMER_MAX_ROLLBACK_NODES];
+static int hrtimer_rollback_cnt = 0;
+
+static void hrtimer_rollback_node(struct node *node)
+{
+	DBG_PRINTF1("Rollback timer properties from node: 0x%p\n", node);
+	/* Remove all hrtimer related properties from the node. */
+	prop_del(node, prop_str_compatible);
+	prop_del(node, prop_str_clocks);
+	prop_del(node, prop_str_tmr_offset);
+	prop_del(node, prop_str_tmr_len);
+	prop_del(node, prop_str_cpu_offset);
+	prop_del(node, prop_str_cs_tmr_idx);
+}
+#endif
 
 int gptimer_fixup_node(ambapp_core *core, struct node *node)
 {
 	/* Use static as core subindex is not generally available. */
 	static int numnode = 0;
+
+#ifdef CONFIG_WATCHDOG
 	ambapp_dev *dev;
 
 	dev = ambapp_get_first_dev(core);
 	if (numnode == 0)
 		node_add_watchdog(node, dev->device, numnode);
+#endif
 
+#ifdef CONFIG_HRTIMERS
+	/* Rollback hrtimer related properties in case an error occurred. */
+	if (!hrtimers_res_ok()) {
+		/* Start with any previously setup nodes */
+		int i;
+		for (i = 0; i < hrtimer_rollback_cnt; i++)
+			hrtimer_rollback_node(hrtimer_rollback_nodes[i]);
+
+		hrtimer_rollback_cnt = 0;
+		/* Rollback the current node */
+		hrtimer_rollback_node(node);
+	} else if (hrtimer_rollback_cnt < HRTIMER_MAX_ROLLBACK_NODES) {
+		/* Store the node in case a future error occurs */
+		hrtimer_rollback_nodes[hrtimer_rollback_cnt++] = node;
+	}
+#endif
 	numnode++;
 
 	return 0;
@@ -295,8 +681,6 @@ int ambapp_prop_uintarray(struct prop_std *buf, const char *prop_str,
 int ambapp_prop_str_array(struct prop_std *buf, const char *prop_str,
 			  const char *val_array[]);
 int ambapp_prop_empty(struct prop_std *buf, const char *prop_str);
-int ambapp_prop_intptr(struct prop_std *buf, const char *prop_str, int *value,
-		       int len);
 
 /* =========================================================================*/
 /* Clock gating */
@@ -1080,6 +1464,11 @@ int grgpio_fixup_core(ambapp_core *core)
 
 void gptimer_init(ambapp_core *core, int level)
 {
+#ifdef CONFIG_HRTIMERS
+	/* Count the number of GPTIMER cores the system */
+	if (level == 0)
+		gptimer_core_cnt++;
+#endif
 	/* Only process the first GPTIMER core */
 	if (gptimer_inited)
 		return;
@@ -1545,7 +1934,7 @@ struct amba_driver amba_drivers[] = {
  /*					   dev, core, prop, init */
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_IRQMP, irqmp_dev, NULL, NULL, NULL, NULL),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_GPTIMER, gptimer_apb_fixup, NULL,    \
-			NULL, gptimer_init, gptimer_fixup_node),
+			gptimer_fixup_prop, gptimer_init, gptimer_fixup_node),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON5, NULL, NULL, NULL, leon_init, NULL),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON4, NULL, NULL, NULL, leon_init, NULL),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON3, NULL, NULL, NULL, leon_init, NULL),
@@ -1930,7 +2319,14 @@ void ahb2ahb_bus_create_iommu(struct ambapp_bus *ainfo, struct ahb_bus *parent,
 	struct iommu_regs *regs = 0;
 	ambapp_ahbdev *pdev;
 	int j;
-	
+
+	if (ainfo->ahb_buses >= AMBA_AHB_MAX) {
+#ifdef CONFIG_DEBUG
+		DBG_PRINTF0("Warning! Detected too many AHB buses. Use -max-ahb-buses to adjust the max number of buses.\n");
+#endif
+		return;
+	}
+
 	/* Add new bus to AMBA Information */
 	subbus = &ainfo->ahbs[ainfo->ahb_buses];
 	subbus->bus_idx = ainfo->ahb_buses++;
@@ -1997,7 +2393,14 @@ void ahb2ahb_bus_create(struct ambapp_bus *ainfo, struct ahb_bus *parent,
 	struct ahb_bus *subbus;
 	unsigned long ioarea;
 	int i;
-	
+
+	if (ainfo->ahb_buses >= AMBA_AHB_MAX) {
+#ifdef CONFIG_DEBUG
+		DBG_PRINTF0("Warning! Detected too many AHB buses. Use -max-ahb-buses to adjust the max number of buses\n");
+#endif
+		return;
+	}
+
 	/* Is there a PnP area behind the bridge?*/
 	ioarea = bridge->userdef[1];
 	if (ioarea == 0)
@@ -2036,6 +2439,13 @@ void ahb2apb_bus_create(struct ambapp_bus *ainfo, struct ahb_bus *parent,
 {
 	struct apb_bus *subbus;
 
+	if (ainfo->apb_buses >= AMBA_APB_MAX) {
+#ifdef CONFIG_DEBUG
+		DBG_PRINTF0("Warning! Detected too many APB buses. Use -max-apb-buses to adjust the max number of buses.\n");
+#endif
+		return;
+	}
+
 	subbus = &ainfo->apbs[ainfo->apb_buses];
 	subbus->bus_idx = ainfo->apb_buses++;
 	subbus->parent_bus_idx = parent->bus_idx;
@@ -2078,10 +2488,24 @@ struct ambapp_bridge_driver bridgedrvs[] = {
 		.type = 0,
 		.create = ahb2ahb_bus_create
 	},
+	{
+		.vendor = VENDOR_GAISLER,
+		.device = GAISLER_APB3MST,
+		.ver_min = 0,
+		.ver_max = 0xff,
+		.type = 1,
+		.create = ahb2apb_bus_create
+	},
 };
 #define AMBAPP_BRIDGE_DRV_NUM \
 		(sizeof(bridgedrvs) / sizeof(struct ambapp_bridge_driver))
 
+static int ambapp_is_apbmst(ambapp_dev *a)
+{
+	return (a->vendor == VENDOR_GAISLER && (a->device == GAISLER_APBMST ||
+						a->device == GAISLER_APB3MST));
+}
+
 
 /************************/
 
@@ -2662,6 +3086,14 @@ void ambapp_cores_init(struct ambapp_bus *ainfo)
 
 struct ambapp_bus ambapp_plb;
 
+/* Early console initialization (before AMBA Plug & Play) */
+static void ambapp_earlycon_init()
+{
+	if (amba_systemid_is_gr740()) {
+		apbuart = (struct apbuart_regs *)GR740_DEBUG_UART;
+	}
+}
+
 /* Initialize AMBA Structures but not bus frequency, pair devices into cores */
 void ambapp_init(void)
 {
@@ -2669,6 +3101,8 @@ void ambapp_init(void)
 	amba_systemid =
 		BYPASS_LOAD_PA(CONFIG_AMBA_IO_AREA | AMBA_CONF_AREA | 0xff0);
 
+	ambapp_earlycon_init();
+
 	ambapp_bus_init(&ambapp_plb, CONFIG_AMBA_IO_AREA);
 
 	ambapp_cores_init(&ambapp_plb);
@@ -3118,15 +3552,6 @@ int (*ambapp_prop_creators[])(ambapp_core *core, struct prop_std *buf) = {
 };
 #define PROP_CREATORS_NUM (sizeof(ambapp_prop_creators)/sizeof(void (*)(void)))
 
-void node_add_child(struct node *node, struct node *child)
-{
-	struct node **np = &node->child;
-
-	while (*np)
-		np = &((*np)->sibling);
-	*np = child;
-}
-
 /**
  * node_has_child: Go through the children of the node and check if there is
  * a child with the expected name.
@@ -3426,7 +3851,7 @@ void ambapp_nodes_init(struct node *b0)
 			case DEV_AHB_MST:
 			case DEV_AHB_SLV:
 				if (core->ahbslv
-				    && (!(a->vendor == VENDOR_GAISLER && a->device == GAISLER_APBMST))
+				    && (!ambapp_is_apbmst(a))
 				    && (b = ambapp_bridge_detect((ambapp_ahbdev *)a))) {
 					/* bridges connect with their slave */
 					a = (ambapp_dev *)core->ahbslv;
