@@ -1,7 +1,7 @@
 /*
  * AMBA Plug'n Play vendor and device name definitions.
  *
- * COPYRIGHT (c) 2024.
+ * COPYRIGHT (c) 2025.
  * Frontgrade Gaisler AB
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
  * Any changes/bug-fixes should be made in the script, then rerun the script
  * to create a new file.
  *
- * Created from GRLIB revision 4293
+ * Created from GRLIB revision 4297
  */
 
 #include <ambapp_names.h>
@@ -254,6 +254,16 @@ static struct ambapp_device_name GAISLER_devices[] =
   {GAISLER_SGMII_VCU118, "SGMII_VCU118", "VCU118 SGMII Interface"},
   {GAISLER_APBUART_16550, "APBUART_16550", "Generic UART 16550"},
   {GAISLER_IOPMP, "IOPMP", "RISC-V IOPMP"},
+  {GAISLER_EFPGA, "EFPGA", "AHB interface to eFPGA28RH_30K"},
+  {GAISLER_GRISOC_MB_EXT, "GRISOC_MB_EXT", "GRISoC Mailbox interface"},
+  {GAISLER_GRISOC_MB_INT, "GRISOC_MB_INT", "GRISoC Mailbox interface (int)"},
+  {GAISLER_GRISOC_BOOT, "GRISOC_BOOT", "GRISoC Boot FSM"},
+  {GAISLER_GRISOC_CRYPTO, "GRISOC_CRYPTO", "GRISoC Crypto accelerator"},
+  {GAISLER_GRISOC_SRNG, "GRISOC_SRNG", "GRISoC Secure RNG"},
+  {GAISLER_GRPPSTIMER, "GRPPSTIMER", "PPS Timer"},
+  {GAISLER_GRLSEDC, "GRLSEDC", "Lattice CRAM Scrubber"},
+  {GAISLER_MDIO_CTRL, "MDIO_CTRL", "Ethernet MDIO Controller"},
+  {GAISLER_OTP_WRPR, "OTP_WRPR", "OTP control wrapper"},
   {0, NULL, NULL}
 };
 
@@ -527,6 +537,16 @@ static struct ambapp_device_name HONEYWELL_devices[] =
 };
 
 
+/* Vendor ST devices */
+static struct ambapp_device_name ST_devices[] =
+{
+  {GAISLER_AHBMST_EM, "AHBMST_EM", "AMBA Master Emulator"},
+  {GAISLER_TEST_1X2, "TEST_1X2", "HAPS TEST_1x2 interface"},
+  {ST_MIPHY, "MIPHY", "MIPHY transceiver"},
+  {0, NULL, NULL}
+};
+
+
 /* Vendor AITECH devices */
 static struct ambapp_device_name AITECH_devices[] =
 {
@@ -541,6 +561,14 @@ static struct ambapp_device_name S3_devices[] =
 };
 
 
+/* Vendor TTTECH devices */
+static struct ambapp_device_name TTTECH_devices[] =
+{
+  {TTTECH_SWITCH, "SWITCH", "TTE Switch"},
+  {0, NULL, NULL}
+};
+
+
 /* Vendor TAS devices */
 static struct ambapp_device_name TAS_devices[] =
 {
@@ -725,8 +753,10 @@ static struct ambapp_ids vendors[] =
   {VENDOR_SEMIBLOCKS, "SEMIBLOCKS", "SemiBlocks B.V.", SEMIBLOCKS_devices},
   {VENDOR_NEC, "NEC", "NEC Corporation", NEC_devices},
   {VENDOR_HONEYWELL, "HONEYWELL", "Honeywell International", HONEYWELL_devices},
+  {VENDOR_ST, "ST", "ST Microelectronics", ST_devices},
   {VENDOR_AITECH, "AITECH", "Aitech Systems", AITECH_devices},
   {VENDOR_S3, "S3", "S3 Group", S3_devices},
+  {VENDOR_TTTECH, "TTTECH", "TTTECH", TTTECH_devices},
   {VENDOR_TAS, "TAS", "Thales Alenia Space", TAS_devices},
   {VENDOR_RECORE, "RECORE", "Recore Systems", RECORE_devices},
   {VENDOR_AAC, "AAC", "AAC Microtec", AAC_devices},
